#include <windows.h>
#include <tchar.h>
#include "assert.h"
#include "objidl.h"
#include "resource.h"

#define CLP_FMT  _T("WDJ Memory Stream")
#define MAX_FROM 80
#define MAX_MSG  255
typedef struct _DLGDATA {
    TCHAR szFrom[MAX_FROM + 1];
    TCHAR szMsg[MAX_MSG + 1];
} DLGDATA;

DLGDATA g_dlgdata; 
const WCHAR *pFile      = L"g:\\temp\\demo.stm";
const WCHAR *pStmFile   = L"dlgdata";

// Foward declarations of functions included in this code module:
LRESULT CALLBACK    DlgProc(HWND, UINT, WPARAM, LPARAM);
int                 WriteDataToStream(LPSTREAM, void*, size_t);
int                 ReadDataFromStream(LPSTREAM, void*, size_t);
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, 
                     int nCmdShow) {
    MSG     msg;
    HWND    hWnd;
    hWnd = CreateDialog(hInstance, (LPCTSTR)IDD_MEMSTREAM_DLG,
                                NULL, (DLGPROC)DlgProc);
    if (hWnd != NULL) {
        ShowWindow(hWnd, nCmdShow);
        UpdateWindow(hWnd);
        while (GetMessage(&msg, NULL, 0, 0))  {
            if (!IsDialogMessage(hWnd, &msg)) {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }
        }
        return msg.wParam;
    }
    return FALSE;
}

#ifdef __BORLANDC__
#   pragma argsused
#endif
LRESULT CALLBACK DlgProc(HWND hDlg, UINT message, WPARAM wParam, 
                         LPARAM lParam) {
    LPSTREAM    pStream;
    HGLOBAL     hGlobal;
    IStorage*   pStorage;

    switch (message) {
    case WM_INITDIALOG:
        memset((void *)&g_dlgdata,0,sizeof(DLGDATA));
        strncpy(g_dlgdata.szFrom,_T("For WDJ"),MAX_FROM);
        strncpy(g_dlgdata.szMsg,_T("This is a test\r\n"),MAX_MSG);
        SetDlgItemText(hDlg,IDC_FROM,g_dlgdata.szFrom);
        SetDlgItemText(hDlg,IDC_MSG,g_dlgdata.szMsg);
        return TRUE;
    case WM_CLOSE: 
        PostQuitMessage(1);
        break;

    case WM_COMMAND:  
        switch (wParam) {
        case IDC_CUT:
            GetDlgItemText(hDlg,IDC_FROM,g_dlgdata.szFrom,MAX_FROM); 
            GetDlgItemText(hDlg,IDC_MSG,g_dlgdata.szMsg,MAX_MSG);
            if (SUCCEEDED(CreateStreamOnHGlobal(NULL,
                                  FALSE,&pStream))) {
                WriteDataToStream(pStream, &g_dlgdata,
                        sizeof(g_dlgdata));
                GetHGlobalFromStream(pStream,&hGlobal); 
                if (OpenClipboard(hDlg)) {
                    SetClipboardData(
                       ::RegisterClipboardFormat(CLP_FMT),hGlobal);
                    CloseClipboard();
                    SetDlgItemText(hDlg,IDC_FROM,_T(""));
                    SetDlgItemText(hDlg,IDC_MSG,_T(""));
                }
            }
            return TRUE;
        case IDC_PASTE:
            if (IsClipboardFormatAvailable(
                    ::RegisterClipboardFormat(CLP_FMT))) 
                if (OpenClipboard(hDlg) ) {
                    if ((hGlobal = (HGLOBAL)GetClipboardData(
                        ::RegisterClipboardFormat(CLP_FMT)))!=NULL)  {
                        if (SUCCEEDED(CreateStreamOnHGlobal(
                                hGlobal,FALSE,&pStream))){
                            ReadDataFromStream(pStream, &g_dlgdata,
                                    sizeof(g_dlgdata));
                            SetDlgItemText(hDlg,IDC_FROM,
                                            g_dlgdata.szFrom);
                            SetDlgItemText(hDlg,IDC_MSG,
                                            g_dlgdata.szMsg);
                            CloseClipboard();
                        }
                    }
                }
            return TRUE;
        case IDC_SAVE:
            GetDlgItemText(hDlg,IDC_FROM,g_dlgdata.szFrom,MAX_FROM); 
            GetDlgItemText(hDlg,IDC_MSG,g_dlgdata.szMsg,MAX_MSG);
            if (SUCCEEDED(StgCreateDocfile(pFile,
                                STGM_READWRITE
                                |STGM_SHARE_EXCLUSIVE
                                |STGM_CREATE,0,&pStorage))) {
               if (SUCCEEDED(pStorage->CreateStream(
                        pStmFile,STGM_WRITE | STGM_CREATE | 
                        STGM_SHARE_EXCLUSIVE ,0,0,&pStream))) {
                    WriteDataToStream(pStream, &g_dlgdata,
                            sizeof(g_dlgdata));
                    pStream->Release();
               }
               pStorage->Release(); 
            }
            break;
        case IDC_LOAD:
            if (SUCCEEDED(StgOpenStorage(pFile,NULL,
                    STGM_READWRITE|STGM_SHARE_EXCLUSIVE,NULL,
                    0,&pStorage))) {
                if (SUCCEEDED(pStorage->OpenStream(pStmFile,
                        0,STGM_READ|STGM_SHARE_EXCLUSIVE ,0,
                        &pStream))) {
                    ReadDataFromStream(pStream, &g_dlgdata,
                            sizeof(g_dlgdata));
                    pStream->Release();
                    SetDlgItemText(hDlg,IDC_FROM,g_dlgdata.szFrom);
                    SetDlgItemText(hDlg,IDC_MSG,g_dlgdata.szMsg);
                }
                pStorage->Release();
            }
            else
                MessageBox(hDlg, "Couldn't open file",
                        "g:\\temp\\demo.stm", MB_OK);
            break;
        case IDC_CLEAR:
            memset(&g_dlgdata,0,sizeof(DLGDATA));
            SetDlgItemText(hDlg,IDC_FROM,g_dlgdata.szFrom);
            SetDlgItemText(hDlg,IDC_MSG,g_dlgdata.szMsg);
            break;                    
        case IDOK:
            EndDialog(hDlg, LOWORD(wParam));
            PostQuitMessage(0);
            return TRUE;
        }
        
        break;
    }
    return FALSE;
}
int WriteDataToStream(LPSTREAM pStream, void* Data, size_t Size) {
    ULONG   nWritten;
    // write the structure to the stream.
    pStream->Write(Data, Size,&nWritten);
    assert(sizeof(_DLGDATA) == nWritten);
    return nWritten;
}
int ReadDataFromStream(LPSTREAM pStream, void* Data, size_t Size) {
    ULONG nWritten;
    pStream->Read(Data, Size, &nWritten);
    assert(sizeof(_DLGDATA) == nWritten);
    return nWritten;
}
